unit FormExports;

interface

uses
  Windows,
  SysUtils, Forms, Controls, StdCtrls, ExtCtrls, CoreExports
  ;

function SelectDeviceViaUI(NumPIDs: LongWord; pPIDs: PLongWord; CaptionStr, FindMultiPromptStr, Find1PromptStr, NoFindPromptStr: PChar; var SelectedDeviceIndex: LongWord): LongWord; cdecl;





implementation

function ErrorText(ErrorCode: LongWord): String;
var
  WindowsError: PChar;

begin
  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER + FORMAT_MESSAGE_FROM_SYSTEM, nil, ErrorCode, 0, PChar(@WindowsError), 0, nil);
  Result := WindowsError;
  LocalFree(Cardinal(WindowsError));
end;



type
  TSelectForm = class(TCustomForm)
  public
    Find1Blurb, FindMultiBlurb, NoFindBlurb: String;
    BlurbLabel: TLabel;
    DeviceList: TListBox;
    ControlPanel: TPanel;
    GoButton, RedetectButton, CancelButton: TButton;
    PIDs: array of LongWord;
    DeviceIndex: LongWord;
    procedure GoButtonClick(Sender: TObject);
    procedure RedetectButtonClick(Sender: TObject);
  end;

{ TSelectForm }

procedure TSelectForm.GoButtonClick(Sender: TObject);
var
  I: Integer;

begin
  I := DeviceList.ItemIndex;
  if I = -1 then begin
    Beep;
    Exit;
  end;

  I := Integer(Pointer(DeviceList.Items.Objects[I]));
  if I = -1 then begin
    Beep;
    Exit;
  end;

  DeviceIndex := I;
  ModalResult := mrOk;
end;

procedure TSelectForm.RedetectButtonClick(Sender: TObject);
var
  DeviceMask, Status: LongWord;
  Index, Valids: Integer;
  PID, NS: LongWord;
  Nom: String;
  I, DI: Integer;

begin
  ClearDevices;
  DeviceList.ItemIndex := -1;
  DeviceList.Clear;
  DeviceIndex := $FFFFFFFF;

  DeviceMask := GetDevices;
  Valids := 0;
  Index := 0;
  while DeviceMask <> 0 do begin
    if (DeviceMask and 1) <> 0 then begin
      NS := 256;
      SetString(Nom, nil, NS);
      Status := QueryDeviceInfo(Index, @PID, @NS, PChar(Nom), nil, nil);
      if Status <> ERROR_SUCCESS then begin
        Nom := '<error ' + IntToStr(Status) + ': ' + ErrorText(Status) + '>';
        DI := -1;
      end
      else begin
        SetLength(Nom, NS);
        DI := -1;
        for I := 0 to High(PIDs) do
          if PID = PIDs[I] then begin
            DI := Index;
            Inc(Valids);
            Break;
          end
        ;
        if DI = -1 then begin
          Nom := Nom + ' <not for this sample>';
        end;
      end;
      Nom := '[' + IntToStr(Index) + '] ' + Nom;
      I := DeviceList.Items.AddObject(Nom, TObject(Pointer(DI)));
      if (DI <> -1) and (DeviceList.ItemIndex = -1) then DeviceList.ItemIndex := I;
    end;

    Inc(Index);
    DeviceMask := DeviceMask shr 1;
  end;

  if Valids = 1 then begin
    GoButton.Enabled := True;
    ActiveControl := GoButton;
    BlurbLabel.Caption := Find1Blurb;
  end
  else if Valids <> 0 then begin
    GoButton.Enabled := True;
    ActiveControl := DeviceList;
    BlurbLabel.Caption := FindMultiBlurb;
  end
  else begin
    GoButton.Enabled := False;
    ActiveControl := RedetectButton;
    BlurbLabel.Caption := NoFindBlurb;
  end;
end;

function SelectDeviceViaUI(NumPIDs: LongWord; pPIDs: PLongWord; CaptionStr, FindMultiPromptStr, Find1PromptStr, NoFindPromptStr: PChar; var SelectedDeviceIndex: LongWord): LongWord; cdecl;
var
  SelectForm: TSelectForm;

begin
  SelectForm := TSelectForm.CreateNew(nil);
  with SelectForm do begin
    Position := poScreenCenter;
    ClientWidth := 192;
    ClientHeight := 256;
    SetLength(PIDs, NumPIDs);
    Move(pPIDs^, PIDs[0], NumPIDs * SizeOf(PIDs[0]));

    if CaptionStr <> nil then Caption := CaptionStr else Caption := 'Select A Device...';
    if FindMultiPromptStr <> nil then FindMultiBlurb := FindMultiPromptStr else FindMultiBlurb := 'Select the device you want to work with and click Go.';
    if Find1PromptStr <> nil then Find1Blurb := Find1PromptStr else Find1Blurb := FindMultiBlurb;
    if NoFindPromptStr <> nil then NoFindBlurb := NoFindPromptStr else NoFindBlurb := 'No compatible devices found; your device may not be installed, or may not be installed properly. Click Redetect to check again.';

    BlurbLabel := TLabel.Create(SelectForm);
    with BlurbLabel do begin
      Parent := SelectForm;
      WordWrap := True;
      Align := alBottom;
      AutoSize := True;
    end;

    DeviceList := TListBox.Create(SelectForm);
    with DeviceList do begin
      Parent := SelectForm;
      Align := alClient;
    end;

    ControlPanel := TPanel.Create(SelectForm);
    with ControlPanel do begin
      Parent := SelectForm;
      Width := 64;
      Align := alRight;
    end;

    GoButton := TButton.Create(SelectForm);
    with GoButton do begin
      Parent := ControlPanel;
      Align := alTop;
      OnClick := GoButtonClick;
      Caption := '&Go';
      Default := True;
    end;

    RedetectButton := TButton.Create(SelectForm);
    with RedetectButton do begin
      Parent := ControlPanel;
      Align := alTop;
      Top := 10000;
      OnClick := RedetectButtonClick;
      Caption := '&Redetect';
    end;

    CancelButton := TButton.Create(SelectForm);
    with CancelButton do begin
      Parent := ControlPanel;
      Align := alTop;
      Top := 10000;
      ModalResult := mrCancel;
      Caption := '&Cancel';
      Cancel := True;
    end;
  end;

  SelectForm.RedetectButton.Click;

  if SelectForm.ShowModal = mrOk then begin
    Result := ERROR_SUCCESS;
    SelectedDeviceIndex := SelectForm.DeviceIndex;
  end
  else begin
    Result := ERROR_CANCELLED;
  end;
  SelectForm.Free;
end;

end.
